@@ -1,3 +1,11 @@
+/*  
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ */
 /*
  * mapscrn.c
  */
@@ -6,6 +14,7 @@
 #include <Tw/Twerrno.h>
 
 #include <string.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
