@@ -1,5 +1,11 @@
 /*
  *  threadtest.c  --  test libTw thread-safety
+ *  Copyright (C) 1993-1999  Massimiliano Ghilardi
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  */
 
@@ -57,7 +63,8 @@ void mainloop(twindow *Win) {
     tevent_any Event;
     /*struct timeval p = {0, 0};*/
     fd_set readfds;
-    long x, y;
+    hwattr x;
+    int y;
 
     FD_ZERO(&readfds);
     srand48(++seed);
@@ -74,7 +81,7 @@ void mainloop(twindow *Win) {
 	for (y = 256; y; y--) {
 	    x = lrand48();
 	    TwWriteHWAttrWindow(*Win, lrand48() >> 26, lrand48() >> 27,
-				sizeof(long)/sizeof(hwattr), (hwattr *)&x);
+				sizeof(long)/sizeof(hwattr), &x);
 	}
 	
 	/* bail out if something goes *really* wrong */
