@@ -1,3 +1,10 @@
+/*  Copyright (C) 2000 by Massimiliano Ghilardi
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ */
 #ifndef _TWIN_UNALIGNED_H
 #define _TWIN_UNALIGNED_H
 
@@ -9,7 +16,7 @@
 
 
 #ifndef TW_CAN_UNALIGNED
-# ifdef __i386__
+# if defined(__i386__) || defined(__x86_64__)
 #  define TW_CAN_UNALIGNED 1
 # else /* !__i386__ */
 #  warning not ix86 arch, using memcpy
