@@ -1,9 +1,19 @@
+/*
+ *
+ *  Copyright (C) 2002 by Massimiliano Ghilardi
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ * 
+ */
 
 #ifndef _TT_ASSERT_H
 #define	_TT_ASSERT_H
 
 /* this prints an "assertion failed" message. */
-void TTAssertFail (char *assertion, char *file, unsigned int line, char *function);
+void TTAssertFail (const char *assertion, const char *file, unsigned int line, const char *function);
 
 # define TTAssertAlways(expr)			\
     ((expr) ? 1 :				\
