@@ -11,6 +11,7 @@
  */
 
 #include "twin.h"
+#include "main.h"
 #include "data.h"
 #include "methods.h"
 #include "hw.h"
@@ -20,9 +21,27 @@
 #include "draw.h"
 
 #ifdef CONF__UNICODE
-# include "Tutf/Tutf.h"
+# include <Tutf/Tutf.h>
+# include <Tutf/Tutf_defs.h>
 #endif
 
+#ifdef CONF__UNICODE
+hwattr extra_POS_INSIDE;
+hwattr extra_POS_ROOT;
+#else
+# define extra_POS_INSIDE 0
+# define extra_POS_ROOT 0
+#endif
+
+
+byte InitDraw(void) {
+#ifdef CONF__UNICODE
+    extra_POS_INSIDE = HWATTR_EXTRA32(0, EncodeToHWAttrExtra(POS_INSIDE, 0, 0, 0));
+    extra_POS_ROOT   = HWATTR_EXTRA32(0, EncodeToHWAttrExtra(POS_ROOT, 0, 0, 0));
+#endif
+    return TRUE;
+}
+
 INLINE hwcol DoShadowColor(hwcol Color, byte Fg, byte Bg) {
     return
 	(
@@ -39,10 +58,10 @@ INLINE hwcol DoShadowColor(hwcol Color, byte Fg, byte Bg) {
  * warning: DrawMenu() can cheat and give us a user Menu
  * while MenuItem is from All->CommonMenu
  */
-void FindFontMenuItem(menu Menu, menuitem MenuItem, dat i, byte Select, hwfont *PtrFont, hwcol *PtrColor) {
+static void FindFontMenuItem(menu Menu, menuitem MenuItem, dat i, byte Select, hwattr *PtrAttr) {
     hwcol Color;
     byte ShortCutFound;
-
+    
     if (Menu && MenuItem && i >= MenuItem->Left && i < MenuItem->Left + MenuItem->Len) {
 	ShortCutFound = i==MenuItem->Left+MenuItem->ShortCut;
 	if (MenuItem->Flags & ROW_ACTIVE) {
@@ -60,12 +79,11 @@ void FindFontMenuItem(menu Menu, menuitem MenuItem, dat i, byte Select, hwfont *
 	else
 	    Color=Menu->ColDisabled;
 
-	*PtrFont=MenuItem->Text[i - MenuItem->Left];
-	*PtrColor=Color;
+	*PtrAttr = HWATTR(Color, MenuItem->Text[i - MenuItem->Left]);
     }
 }
 
-void FindFontInfo(menu Menu, dat i, byte Select, hwfont *PtrFont, hwcol *PtrColor) {
+static void FindFontInfo(menu Menu, dat i, byte Select, hwattr *PtrAttr) {
     row Info;
     hwcol Color;
     
@@ -79,9 +97,7 @@ void FindFontInfo(menu Menu, dat i, byte Select, hwfont *PtrFont, hwcol *PtrColo
 	else
 	    Color = Info->ColText[i];
 	
-	/* they are both hwfont, no translation required */
-	*PtrFont=Info->Text[i];
-	*PtrColor=Color;
+	*PtrAttr = HWATTR(Color, Info->Text[i]);
     }
 }
 
@@ -109,7 +125,7 @@ void DrawDesktop(screen Screen, dat X1, dat Y1, dat X2, dat Y2, byte Shaded) {
     Y1 = Max2(Y1, YLimit+1);
     X2 = Min2(X2, DWidth-1);
     Y2 = Min2(Y2, DHeight-1);
-    
+
     if (Screen && S_USE(Screen, USEBG)) {
 	Attr = Screen->USE.B.Bg;
 	BgWidth = Screen->USE.B.BgWidth;
@@ -134,16 +150,19 @@ void DrawDesktop(screen Screen, dat X1, dat Y1, dat X2, dat Y2, byte Shaded) {
 		
 	    if (Shaded) {
 		for (X = X1; X <= X2; X++, x++) {
-		    if (x >= BgWidth) x -= BgWidth;
-		    
+		    if (x >= BgWidth)
+			x -= BgWidth;
 		    attr = Attr[x + y];
 		    col = DoShadowColor(HWCOL(attr), Shaded, Shaded);
-		    Video[X + Y] = HWATTR(col, HWFONT(attr));
+
+		    Video[X + Y] = HWATTR(col, HWFONT(attr)) | extra_POS_ROOT;
 		}
 	    } else {
 		for (X = X1; X <= X2; X++, x++) {
-		    if (x >= BgWidth) x -= BgWidth;
-		    Video[X + Y] = Attr[x + y];
+		    if (x >= BgWidth)
+			x -= BgWidth;
+		    
+		    Video[X + Y] = Attr[x + y] | extra_POS_ROOT;
 		}
 	    }
 	}
@@ -160,11 +179,10 @@ void DrawDesktop(screen Screen, dat X1, dat Y1, dat X2, dat Y2, byte Shaded) {
 	    col = DoShadowColor(HWCOL(attr), Shaded, Shaded);
 	    attr = HWATTR(col, 0) | HWATTR_FONTMASK(attr);
 	}
-	FillVideo(X1, Y1, X2, Y2, attr);
+	FillVideo(X1, Y1, X2, Y2, attr | extra_POS_ROOT);
     }
 }
 
-
 /*
  * initialize a draw_ctx given widget-relative coords
  * (0,0 is the widget top-left corner)
@@ -198,6 +216,10 @@ byte InitDrawCtx(widget W, dat X1, dat Y1, dat X2, dat Y2, byte Shaded, draw_ctx
     D->Y2  = Y2;
 
     while (W) {
+	
+	if (W->Flags & WIDGETFL_NOTVISIBLE)
+	    return FALSE;
+	
 	HasTopBar = HasBorder = 0;
 
 	if (!IS_WINDOW(W) || !(((window)W)->Attrib & WINDOW_ROLLED_UP))
@@ -330,6 +352,10 @@ widget FindWidgetAt(widget Parent, dat X, dat Y) {
     }
     
     for (W = Parent->FirstW; W; W = W->Next) {
+	
+	if (W->Flags & WIDGETFL_NOTVISIBLE)
+	    continue;
+	
 	i = X + Parent->XLogic;
 	j = Y + Parent->YLogic;
 
@@ -368,13 +394,11 @@ widget RecursiveFindWidgetAt(widget Parent, dat X, dat Y) {
 
 
 void DrawSelfWidget(draw_ctx *D) {
-    widget W;
+    widget W = D->TopW;
     
-    if (QueuedDrawArea2FullScreen)
+    if (QueuedDrawArea2FullScreen || (W->Flags & WIDGETFL_NOTVISIBLE))
 	return;
     
-    W = D->TopW;
-
     if (w_USE(W, USEEXPOSE)) {
 	CONST byte *Text = NULL;
 	CONST hwfont *HWFont = NULL;
@@ -384,6 +408,7 @@ void DrawSelfWidget(draw_ctx *D) {
 	dat Pitch, X1, X2, Y1, Y2, dX, dY;
 	ldat _X1, _X2, _Y1, _Y2;
 	dat DWidth, i, j;
+	hwattr h;
 	hwcol Color;
 	byte Shaded;
 
@@ -427,12 +452,13 @@ void DrawSelfWidget(draw_ctx *D) {
 	    _Y1 = Max2(W->USE.E.Y1 + Up, Y1);
 	    _Y2 = Min2(W->USE.E.Y2 + Up, Y2);
 
-	    dX = _X1 - W->USE.E.X1 + Left;
-	    dY = _Y1 - W->USE.E.Y1 + Up;
+	    dX = _X1 - W->USE.E.X1 - Left;
+	    dY = _Y1 - W->USE.E.Y1 - Up;
 	    
+	    h = W->USE_Fill | extra_POS_INSIDE;
 	    if (_X1 > _X2 || _Y1 > _Y2) {
 		/* no valid ->USE.E, fill with spaces */
-		FillVideo(X1, Y1, X2, Y2, W->USE_Fill);
+		FillVideo(X1, Y1, X2, Y2, h);
 		return;
 	    }
 
@@ -443,19 +469,19 @@ void DrawSelfWidget(draw_ctx *D) {
 	     * pad with SPACEs as needed
 	     */
 	    if (Y1 < _Y1) {
-		FillVideo(X1, Y1, X2, _Y1 - 1, W->USE_Fill);
+		FillVideo(X1, Y1, X2, _Y1 - 1, h);
 		Y1 = _Y1;
 	    }
 	    if (Y2 > _Y2) {
-		FillVideo(X1, _Y2 + 1, X2, Y2, W->USE_Fill);
+		FillVideo(X1, _Y2 + 1, X2, Y2, h);
 		Y2 = _Y2;
 	    }
 	    if (X1 < _X1) {
-		FillVideo(X1, Y1, _X1 - 1, Y2, W->USE_Fill);
+		FillVideo(X1, Y1, _X1 - 1, Y2, h);
 		X1 = _X1;
 	    }
 	    if (X2 > _X2) {
-		FillVideo(_X2 + 1, Y1, X2, Y2, W->USE_Fill);
+		FillVideo(_X2 + 1, Y1, X2, Y2, h);
 		X2 = _X2;
 	    }
 	    
@@ -504,7 +530,7 @@ void DrawSelfWidget(draw_ctx *D) {
 	    msg Msg;
 	    event_widget *EventW;
 
-	    if ((Msg=Do(Create,Msg)(FnMsg, MSG_WIDGET_CHANGE, sizeof(event_widget)))) {
+	    if ((Msg=Do(Create,Msg)(FnMsg, MSG_WIDGET_CHANGE, 0))) {
 		EventW = &Msg->Event.EventWidget;
 		EventW->W      = W;
 		EventW->Code   = MSG_WIDGET_EXPOSE;
@@ -517,18 +543,16 @@ void DrawSelfWidget(draw_ctx *D) {
 	    }
 	}
     } else
-	FillVideo(D->X1, D->Y1, D->X2, D->Y2, W->USE_Fill);
+	FillVideo(D->X1, D->Y1, D->X2, D->Y2, W->USE_Fill | extra_POS_INSIDE);
 }
 
 
 void DrawSelfGadget(draw_ctx *D) {
-    gadget G;
+    gadget G = (gadget)D->TopW;
 
-    if (QueuedDrawArea2FullScreen)
+    if (QueuedDrawArea2FullScreen || (G->Flags & WIDGETFL_NOTVISIBLE))
 	return;
     
-    G = (gadget)D->TopW;
-    
     if (!G_USE(G, USETEXT)) {
 	G->Fn->Fn_Widget->DrawSelf(D);
 	return;
@@ -606,7 +630,7 @@ void DrawSelfGadget(draw_ctx *D) {
 		if (!Absent)
 		    Color = DoShadowColor(ColText[i + j * width], D->Shaded, D->Shaded);
 		Video[i + j * DWidth + Offset] =
-		    HWATTR(Color, Font);
+		    HWATTR(Color, Font) | extra_POS_INSIDE;
 	    }
 	}
 	DirtyVideo(D->X1, D->Y1, D->X2, D->Y2);
@@ -619,18 +643,18 @@ static void DrawSelfBorder(window Window, ldat Left, ldat Up, ldat Rgt, ldat Dwn
 
     dat i, j, u, v;
     dat DWidth = All->DisplayWidth;
-    hwfont Font;
+    hwattr Attr;
     hwcol Color;
     
-    if (QueuedDrawArea2FullScreen)
+    if (QueuedDrawArea2FullScreen || (Window->Flags & WIDGETFL_NOTVISIBLE))
 	return;
     
     if ((ldat)Y1==Up) {
 	j=Y1*DWidth;
 	for (i=X1, u=(ldat)i-Left; i<=X2; i++, u++) {
-	    Act(FindBorder,Window)(Window, u, 0, Border, &Font, &Color);
-	    Color=DoShadowColor(Color, Shaded || !WinActive, Shaded);
-	    Video[i+j]=HWATTR(Color, Font);
+	    Act(FindBorder,Window)(Window, u, 0, Border, &Attr);
+	    Color = DoShadowColor(HWCOL(Attr), Shaded || !WinActive, Shaded);
+	    Video[i+j] = HWATTR(Color, 0) | (Attr & ~HWATTR(MAXHWCOL, 0));
 	}
 	DirtyVideo(X1, Y1, X2, Y1);
 	Y1++;
@@ -642,9 +666,9 @@ static void DrawSelfBorder(window Window, ldat Left, ldat Up, ldat Rgt, ldat Dwn
 	v=((ldat)Y2-Up);
 	j=Y2*DWidth;
 	for (i=X1, u=(ldat)i-Left; i<=X2; i++, u++) {
-	    Act(FindBorder,Window)(Window, u, v, Border, &Font, &Color);
-	    Color=DoShadowColor(Color, Shaded || !WinActive, Shaded);
-	    Video[i+j]=HWATTR(Color, Font);
+	    Act(FindBorder,Window)(Window, u, v, Border, &Attr);
+	    Color = DoShadowColor(HWCOL(Attr), Shaded || !WinActive, Shaded);
+	    Video[i+j] = HWATTR(Color, 0) | (Attr & ~HWATTR(MAXHWCOL, 0));
 	}
 	DirtyVideo(X1, Y2, X2, Y2);
 	Y2--;
@@ -654,9 +678,9 @@ static void DrawSelfBorder(window Window, ldat Left, ldat Up, ldat Rgt, ldat Dwn
     
     if ((ldat)X1==Left) {
 	for (j=Y1, v=(ldat)j-Up; j<=Y2; j++, v++) {
-	    Act(FindBorder,Window)(Window, 0, v, Border, &Font, &Color);
-	    Color=DoShadowColor(Color, Shaded || !WinActive, Shaded);
-	    Video[X1+j*DWidth]=HWATTR(Color, Font);
+	    Act(FindBorder,Window)(Window, 0, v, Border, &Attr);
+	    Color = DoShadowColor(HWCOL(Attr), Shaded || !WinActive, Shaded);
+	    Video[X1+j*DWidth] = HWATTR(Color, 0) | (Attr & ~HWATTR(MAXHWCOL, 0));
 	}
 	DirtyVideo(X1, Y1, X1, Y2);
 	X1++;
@@ -667,9 +691,9 @@ static void DrawSelfBorder(window Window, ldat Left, ldat Up, ldat Rgt, ldat Dwn
     if ((ldat)X2==Rgt) {
 	u=((ldat)X2-Left);
 	for (j=Y1, v=(ldat)j-Up; j<=Y2; j++, v++) {
-	    Act(FindBorder,Window)(Window, u, v, Border, &Font, &Color);
-	    Color=DoShadowColor(Color, Shaded || !WinActive, Shaded);
-	    Video[X2+j*DWidth]=HWATTR(Color, Font);
+	    Act(FindBorder,Window)(Window, u, v, Border, &Attr);
+	    Color = DoShadowColor(HWCOL(Attr), Shaded || !WinActive, Shaded);
+	    Video[X2+j*DWidth] = HWATTR(Color, 0) | (Attr & ~HWATTR(MAXHWCOL, 0));
 	}
 	DirtyVideo(X2, Y1, X2, Y2);
 	X2--;
@@ -679,13 +703,11 @@ static void DrawSelfBorder(window Window, ldat Left, ldat Up, ldat Rgt, ldat Dwn
 }
 
 void DrawSelfWindow(draw_ctx *D) {
-    window W;
+    window W = (window)D->TopW;
 
-    if (QueuedDrawArea2FullScreen)
+    if (QueuedDrawArea2FullScreen || (W->Flags & WIDGETFL_NOTVISIBLE))
 	return;
 
-    W = (window)D->TopW;
-
     if (!W_USE(W, USEROWS) && !W_USE(W, USECONTENTS)) {
 	W->Fn->Fn_Widget->DrawSelf(D);
 	return;
@@ -735,13 +757,13 @@ void DrawSelfWindow(draw_ctx *D) {
 	    if (Y2 - Up >= W->HLogic) {
 		/* the ->Contents buffer is smaller than the window size... pad with SPACEs */
 		dat Ynew = Up + W->HLogic;
-		FillVideo(X1, Ynew, X2, Y2, HWATTR(W->ColText, ' '));
+		FillVideo(X1, Ynew, X2, Y2, HWATTR(W->ColText, ' ') | extra_POS_INSIDE);
 		Y2 = Ynew - 1;
 	    }
 	    if (X2 - Left >= W->WLogic) {
 		/* the ->Contents buffer is smaller than the window size... pad with SPACEs */
 		dat Xnew = Left + W->WLogic;
-		FillVideo(Xnew, Y1, X2, Y2, HWATTR(W->ColText, ' '));
+		FillVideo(Xnew, Y1, X2, Y2, HWATTR(W->ColText, ' ') | extra_POS_INSIDE);
 		X2 = Xnew - 1;
 	    }
 	    if (X1 <= X2 && Y1 <= Y2) {
@@ -771,7 +793,8 @@ void DrawSelfWindow(draw_ctx *D) {
 				    Color = W->ColSelect;
 				else
 				    Color = HWCOL(CurrCont[v]);
-				Video[i+j*DWidth] = HWATTR(Color, HWFONT(CurrCont[v]));
+				
+				Video[i+j*DWidth] = HWATTR(Color, HWFONT(CurrCont[v])) | extra_POS_INSIDE;
 			    }
 			}
 			CurrCont += W->WLogic;
@@ -792,9 +815,9 @@ void DrawSelfWindow(draw_ctx *D) {
 				Color = W->ColSelect;
 			    else
 				Color = HWCOL(CurrCont[v]);
-			    Color=DoShadowColor(Color, Shaded, Shaded);
+			    Color = DoShadowColor(Color, Shaded, Shaded);
 			    
-			    Video[i+j*DWidth] = HWATTR(Color, HWFONT(CurrCont[v]));
+			    Video[i+j*DWidth] = HWATTR(Color, HWFONT(CurrCont[v])) | extra_POS_INSIDE;
 			}
 			CurrCont += W->WLogic;
 			if (!Row)
@@ -840,7 +863,15 @@ void DrawSelfWindow(draw_ctx *D) {
 		
 		for (i=X1; i<=X2; i++, PosInRow++) {
 		    
-		    if ((Absent=(!CurrRow || PosInRow>=CurrRow->Len)))
+		    Absent = (!CurrRow || PosInRow>=CurrRow->Len);
+		    
+		    if (CurrRow && IS_MENUITEM(CurrRow) && ((menuitem)CurrRow)->Window && i == Rgt) {
+#ifdef CONF__UNICODE
+			Font = T_UTF_16_BLACK_RIGHT_POINTING_TRIANGLE;
+#else
+			Font = '\x10'; /* T_IBM437_BLACK_RIGHT_POINTING_TRIANGLE */
+#endif
+		    } else if (Absent)
 			Font = ' ';
 		    else
 			Font = HWFont[PosInRow];
@@ -867,7 +898,7 @@ void DrawSelfWindow(draw_ctx *D) {
 			Color = ColText[PosInRow];
 		    
 		    Color=DoShadowColor(Color, Shaded, Shaded);
-		    Video[i+j*DWidth]=HWATTR(Color, Font);
+		    Video[i+j*DWidth] = HWATTR(Color, Font) | extra_POS_INSIDE;
 		}
 		if (CurrRow) {
 		    W->USE.R.RowSplit = CurrRow;
@@ -880,7 +911,7 @@ void DrawSelfWindow(draw_ctx *D) {
 	    /* either an unknown window type or just one of the above, but empty */
 	    Color = W->ColText;
 	    Color = DoShadowColor(Color, Shaded, Shaded);
-	    FillVideo(X1, Y1, X2, Y2, HWATTR(Color, ' '));
+	    FillVideo(X1, Y1, X2, Y2, HWATTR(Color, ' ') | extra_POS_INSIDE);
 	}
     }
 }
@@ -1012,7 +1043,7 @@ static void DrawWCtx(draw_ctx *D) {
 		    cR =       cL + ChildNext->XWidth-1;
 		    cU = Up   + ChildNext->Up;
 		    cD =       cU + ChildNext->YWidth-1;
-		    if (cL>X2 || cU>Y2 || cR<X1 || cD<Y1)
+		    if (cL>X2 || cU>Y2 || cR<X1 || cD<Y1 || (ChildNext->Flags & WIDGETFL_NOTVISIBLE))
 			ChildNext=ChildNext->Next;
 		    else
 			ChildFound=TRUE;
@@ -1280,6 +1311,10 @@ static void DrawAreaCtx(draw_ctx *D) {
 
 	WidgetFound=FALSE;
 	while (W && !WidgetFound) {
+	    if (W->Flags & WIDGETFL_NOTVISIBLE) {
+		W=W->Next;
+		continue;
+	    }
 	    shLeft = (ldat)W->Left + (ldat)FirstScreen->dummyLeft - FirstScreen->XLogic;
 	    shUp   = (ldat)W->Up + (ldat)YLimit - FirstScreen->YLogic;
 	    shRgt=shLeft + (ldat)W->XWidth-1;
@@ -1524,7 +1559,8 @@ void DrawBorderWindow(window Window, byte Flags) {
     widget Parent;
     screen Screen, FirstScreen;
 
-    if (QueuedDrawArea2FullScreen || !Window || (Window->Flags & WINDOWFL_BORDERLESS) ||
+    if (QueuedDrawArea2FullScreen || !Window ||
+	(Window->Flags & (WINDOWFL_BORDERLESS|WINDOWFL_NOTVISIBLE)) ||
 	!(Parent=Window->Parent) || !IS_SCREEN(Parent))
 	return;
 
@@ -1573,7 +1609,8 @@ void DrawAreaShadeWindow(screen Screen, window Window, dat X1, dat Y1, dat X2, d
     setup *SetUp;
     
     SetUp=All->SetUp;
-    if (QueuedDrawArea2FullScreen || !Window || !Screen ||
+    if (QueuedDrawArea2FullScreen || !Window ||
+	(Window->Flags & WINDOWFL_NOTVISIBLE) || !Screen ||
 	X1>X2 || Y1>Y2 || !(SetUp->Flags & SETUP_SHADOWS))
 	return;
     
@@ -1642,6 +1679,7 @@ void DrawShadeWindow(window Window, dat X1, dat Y1, dat X2, dat Y2, byte Interna
 }
 
 
+
 /* replaces DrawAreaWindow() */
 void DrawAreaWindow2(window W) {
     draw_ctx D;
@@ -1684,7 +1722,8 @@ void DrawLogicWidget(widget W, ldat X1, ldat Y1, ldat X2, ldat Y2) {
     ldat XL, YL;
     byte HasBorder;
     
-    if (!QueuedDrawArea2FullScreen && W && (!IS_WINDOW(W) || !(((window)W)->Attrib & WINDOW_ROLLED_UP)) && X2>=X1 && Y2>=Y1) {
+    if (!QueuedDrawArea2FullScreen && W && !(W->Flags & WIDGETFL_NOTVISIBLE) &&
+	(!IS_WINDOW(W) || !(((window)W)->Attrib & WINDOW_ROLLED_UP)) && X2>=X1 && Y2>=Y1) {
 	XL = W->XLogic;
 	YL = W->YLogic;
     
@@ -1715,7 +1754,9 @@ void ReDrawRolledUpAreaWindow(window Window, byte Shaded) {
     dat DWidth, DHeight;
     dat YLimit;
     
-    if (QueuedDrawArea2FullScreen || !Window || !Window->Parent || !IS_SCREEN(Window->Parent))
+    if (QueuedDrawArea2FullScreen || !Window ||
+	(Window->Flags & WINDOWFL_NOTVISIBLE) ||
+	!Window->Parent || !IS_SCREEN(Window->Parent))
 	return;
 
     Screen = (screen)Window->Parent;
@@ -1744,6 +1785,112 @@ void ReDrawRolledUpAreaWindow(window Window, byte Shaded) {
     DrawArea2((screen)0, (widget)0, (widget)0, (dat)shLeft, (dat)shUp, (dat)shRgt, (dat)shDwn, Shaded);
 }
 
+
+void DrawMenuScreen(screen Screen, dat Xstart, dat Xend) {
+#ifdef CONF__UNICODE
+    static hwattr extra;
+#endif
+    screen fScreen;
+    menu Menu;
+    menuitem Item;
+    dat DWidth, DHeight, i, j, x;
+    hwattr Attr;
+    hwfont Font;
+    hwcol Color;
+    byte Select, State, MenuInfo;
+    
+    if (QueuedDrawArea2FullScreen || !Screen || !Screen->All || Xstart>Xend)
+	return;
+    
+    j = Screen->YLimit;
+    DWidth = All->DisplayWidth;
+    DHeight = All->DisplayHeight;
+    
+    if (j<0 || j>=DHeight || Xstart>=DWidth || Xstart > Xend)
+	return;
+    
+    for (fScreen = All->FirstScreen; fScreen && fScreen != Screen; fScreen = fScreen->Next) {
+	if (fScreen->YLimit <= j)
+	    return;
+    }
+    if (fScreen != Screen)
+	return;
+	
+    State = All->State & STATE_ANY;
+    Menu = Act(FindMenu,Screen)(Screen);
+    
+    MenuInfo = State != STATE_MENU && (All->SetUp->Flags & SETUP_MENU_INFO);
+    
+    Xstart = Max2(Xstart, 0);
+    Xend   = Min2(Xend, DWidth-1);
+    
+
+#ifdef CONF__UNICODE	
+    if (!extra)
+	extra = EncodeToHWAttrExtra(POS_MENU, 0, 0, 0);
+#endif
+
+    for (i=Xstart; i<=Xend; i++) {
+	if (i+2>=DWidth) {
+	    Color = State == STATE_SCREEN ? Menu->ColSelShtCut : Menu->ColShtCut;
+	    if (XAND(Screen->Flags, SCREENFL_BACK_SELECT|SCREENFL_BACK_PRESSED))
+		Color = COL( COLBG(Color), COLFG(Color));
+	    Font = Screen_Back[2-(DWidth-i)];
+	}
+	else if (DWidth-i<=(dat)3+lenTWDisplay) {
+	    Color = State == STATE_SCREEN ? Menu->ColSelShtCut : Menu->ColShtCut;
+	    Font = TWDisplay[3 + lenTWDisplay - (DWidth - i)];
+	    if (!Font) Font = ' ';
+	}
+	else if (DWidth-i>(dat)9 && DWidth-i<=(dat)9+All->BuiltinRow->Len) {
+	    Color = State == STATE_SCREEN ? Menu->ColSelect : Menu->ColItem;
+	    Font = All->BuiltinRow->Text[9+All->BuiltinRow->Len - (DWidth-i)];
+	    if (!Font) Font = ' ';
+	}
+	else if (MenuInfo && FindInfo(Menu, i)) {
+	    Select = State == STATE_SCREEN;
+	    FindFontInfo(Menu, i, Select, &Attr);
+	    Font = HWFONT(Attr);
+	    Color = HWCOL(Attr);
+	}
+	else {
+	    if (!MenuInfo) {
+		Item = Act(FindItem,Menu)(Menu, i);
+		
+		if (Item) {
+		    /* check if Item is from All->CommonMenu */
+		    if ((menu)Item->Parent == All->CommonMenu && Menu->LastI)
+			x = Menu->LastI->Left + Menu->LastI->Len;
+		    else
+			x = 0;
+
+		    Select = State == STATE_SCREEN ||
+			(State == STATE_MENU && ((menu)Item->Parent)->SelectI == Item);
+		    /*
+		     * CHEAT: Item may be in CommonMenu, not in Menu...
+		     * steal Item color from Menu.
+		     */
+		    FindFontMenuItem(Menu, Item, i - x, Select, &Attr);
+		    Font = HWFONT(Attr);
+		    Color = HWCOL(Attr);
+		}
+	    }
+	    if (MenuInfo || !Item) {
+		Color = State == STATE_SCREEN ? Menu->ColSelect : Menu->ColItem;
+		Font = ' ';
+	    }
+	}
+	if (Screen != All->FirstScreen)
+	    Color = Menu->ColDisabled;
+#ifdef CONF__UNICODE
+	Video[i+j*DWidth]=HWATTR_EXTRA32(HWATTR(Color, Font), extra);
+#else
+	Video[i+j*DWidth]=HWATTR(Color, Font);
+#endif
+    }
+    DirtyVideo(Xstart, j, Xend, j);    
+}
+
 void DrawScreen(screen Screen) {
     screen FirstScreen;
     if (!QueuedDrawArea2FullScreen) {
