@@ -66,7 +66,7 @@
  * 
  * Focus [On|Off|Toggle]
  * 
- * GlobalFlags [[+|-]AltFont] [[+|-]AlwaysCursor] [[+|-]Blink] [[+|-]EdgeScroll] [[+|-]HideMenu] [[+|-]MenuItems] [[+|-]Shadows] [Shadows <x> <y>] [SelectionButton <n>]
+ * GlobalFlags [[+|-]AltFont] [[+|-]CursorAlways] [[+|-]Blink] [[+|-]ScreenScroll] [[+|-]MenuHide] [[+|-]MenuItems] [[+|-]Shadows] [Shadows <x> <y>] [ButtonSelection <n>]
  * 
  * Interactive <kind>
  * # Scroll, Move, Resize, Screen, Menu
@@ -311,8 +311,8 @@ static ldat FreeButtonPos(ldat n, ldat lr) {
 static byte ImmButton(ldat n, str shape, ldat lr, ldat flag, ldat pos) {
     if (n >= 0 && n < BUTTON_MAX && strlen(shape) >= 2) {
 #ifdef CONF__UNICODE
-	All->ButtonVec[n].shape[0] = Tutf_IBM437_to_UTF_16[shape[0]];
-	All->ButtonVec[n].shape[1] = Tutf_IBM437_to_UTF_16[shape[1]];
+	All->ButtonVec[n].shape[0] = Tutf_CP437_to_UTF_16[shape[0]];
+	All->ButtonVec[n].shape[1] = Tutf_CP437_to_UTF_16[shape[1]];
 #else
 	All->ButtonVec[n].shape[0] = shape[0];
 	All->ButtonVec[n].shape[1] = shape[1];
@@ -371,14 +371,15 @@ static byte ImmGlobalFlags(node l) {
     while (l) {
 	switch (l->id) {
 	  case ALTFONT:      /*ignored for compatibility*/ return TRUE;
-	  case ALWAYSCURSOR: i = SETUP_ALWAYSCURSOR; break;
 	  case BLINK:        i = SETUP_BLINK;        break;
-	  case EDGESCROLL:   i = SETUP_EDGESCROLL;   break;
-	  case HIDEMENU:     i = SETUP_HIDEMENU;     break;
-	  case MENUINFO:     i = SETUP_MENUINFO;     break;
+	  case CURSOR_ALWAYS:i = SETUP_CURSOR_ALWAYS;break;
+	  case SCREEN_SCROLL:i = SETUP_SCREEN_SCROLL;break;
+	  case MENU_HIDE:    i = SETUP_MENU_HIDE;    break;
+	  case MENU_INFO:    i = SETUP_MENU_INFO;    break;
+	  case MENU_RELAX:   i = SETUP_MENU_RELAX;   break;
 	  case SHADOWS:      i = SETUP_SHADOWS;      break;
-	  case PASTEBUTTON:  i = 0;                  break;
-	  case SELECTIONBUTTON: i = -1;              break;
+	  case BUTTON_PASTE:  i = 0;                  break;
+	  case BUTTON_SELECTION: i = -1;              break;
 	  default:           return FALSE;
 	}
 	if (i > 0) switch (l->x.f.flag) {
@@ -405,10 +406,10 @@ static byte ImmGlobalFlags(node l) {
 	  default:
 	    return FALSE;
 	} else {
-	    /* SelectionButton */
+	    /* ButtonSelection or ButtonPaste */
 
 	    j = l->x.f.flag;
-	    if (j >= 1 && j <= 3)
+	    if (j >= 1 && j <= BUTTON_N_MAX)
 		GlobalFlags[i+3] = HOLD_CODE(j-1);
 	    else
 		GlobalFlags[i+3] = HOLD_LEFT;
@@ -498,18 +499,18 @@ static byte BindMouse(str buttons, str _ctx, node func) {
     byte c;
     
     while ((c = *buttons++)) {
-	if (c >= '1' && c <= '3')
+	if (c >= '1' && c <= '1' + BUTTON_N_MAX)
 	    buttonmask |= HOLD_CODE(c-'1');
 	else if (c == 'H' || c == 'h')
-	    buttonmask |= HOLD_CODE(3);
+	    buttonmask |= PRESS_;
 	else if (c == 'C' || c == 'c')
-	    buttonmask |= HOLD_CODE(4);
+	    buttonmask |= RELEASE_;
     }
     if (buttonmask) {
-	if (!(buttonmask & (HOLD_CODE(3)|HOLD_CODE(4))))
+	if (!(buttonmask & (PRESS_|RELEASE_)))
 	    /* default is 'C' */
-	    buttonmask |= HOLD_CODE(4);
-	
+	    buttonmask |= RELEASE_;
+
 	if (!(n = LookupBind(buttonmask, ctx, MouseList))) {
 	    n = MakeBuiltinFunc(buttonmask);
 	    n->x.ctx = ctx;
@@ -529,7 +530,12 @@ static node MakeFlagNode(ldat id, ldat flag) {
 }
 
 static node MakeModuleNode(str label, ldat flag) {
-    node n = MakeNode(label);
+    node n;
+    ldat len = strlen(label);
+    /* remove .so suffix */
+    if (len > 3 && !memcmp(label+len-3, ".so", 3))
+	label[len-3] = '\0';
+    n = MakeNode(label);
     n->id = MODULE;
     n->x.f.flag = flag;
     n->x.f.a = -1; /* code for this module will be asked when needed */
@@ -619,7 +625,7 @@ static node MakeSendToScreen(str name) {
 
 static node MakeSyntheticKey(ldat shiftflags, str label) {
     ldat key;
-    str  seq;
+    CONST byte *seq;
     byte buf[4];
     node n;
     
@@ -696,6 +702,10 @@ static node AddtoStringList(node l, str string) {
 
 static str TokenName(ldat id) {
     switch (id) {
+      case BLACK:	return "BLACK";
+      case BLUE:	return "BLUE";
+      case GREEN:	return "GREEN";
+
       case '+':		return "+";
       case '-':		return "-";
       case ADDSCREEN:	return "AddScreen";
@@ -755,14 +765,15 @@ static str TokenName(ldat id) {
       case FL_LEFT:	return "Left";
       case FL_RIGHT:	return "Right";
       case ALTFONT:	return "AltFont";
-      case ALWAYSCURSOR:return "AlwaysCursor";
       case BLINK:	return "Blink";
-      case EDGESCROLL:	return "EdgeScroll";
-      case HIDEMENU:	return "HideMenu";
-      case MENUINFO:	return "MenuInfo";
+      case CURSOR_ALWAYS:return "CursorAlways";
+      case MENU_HIDE:	return "MenuHide";
+      case MENU_INFO:	return "MenuInfo";
+      case MENU_RELAX:	return "MenuRelax";
+      case SCREEN_SCROLL:return "ScreenScroll";
       case SHADOWS:	return "Shadows";
-      case PASTEBUTTON: return "PasteButton";
-      case SELECTIONBUTTON: return "SelectionButton";
+      case BUTTON_PASTE:return "ButtonPaste";
+      case BUTTON_SELECTION:return "ButtonSelection";
       default:		break;
     }
     return "(unknown)";
@@ -791,16 +802,16 @@ static str ColorName(hwcol col) {
 static void DumpColorName(hwcol col) {
     hwcol fg = COLFG(col), bg = COLBG(col);
     
-    if (fg & HIGH) printf("%s ", ColorName(HIGH));
-    printf("%s %s ", ColorName(fg & ~HIGH), TokenName(FL_ON));
-    if (bg & HIGH) printf("%s ", ColorName(HIGH));
-    printf("%s ", ColorName(bg & ~HIGH));
+    if (fg & HIGH) fprintf(stderr, "%s ", ColorName(HIGH));
+    fprintf(stderr, "%s %s ", ColorName(fg & ~HIGH), TokenName(FL_ON));
+    if (bg & HIGH) fprintf(stderr, "%s ", ColorName(HIGH));
+    fprintf(stderr, "%s ", ColorName(bg & ~HIGH));
 }
 
 static void DumpNameList(node l, byte nl) {
     for (; l; l = l->next) {
 	if (l->name)
-	    printf("\"%s\"%c", l->name, nl ? '\n' : ' ');
+	    fprintf(stderr, "\"%s\"%c", l->name, nl ? '\n' : ' ');
     }
 }
 
@@ -809,17 +820,17 @@ static void DumpGenericNode(node n) {
 
     if (n) {
 	if (n->id)
-	    printf("%s ", TokenName(n->id));
+	    fprintf(stderr, "%s ", TokenName(n->id));
 	if (n->name)
-	    printf("\"%s\" ", n->name);
+	    fprintf(stderr, "\"%s\" ", n->name);
 	if ((f = n->x.f.plus_minus))
-	    printf("%s", TokenName(f));
+	    fprintf(stderr, "%s", TokenName(f));
 	if ((x = n->x.f.a) || f == '+' || f == '-')
-	    printf("%d ", x);
+	    fprintf(stderr, "%d ", x);
 	if ((f = n->x.f.flag))
-	    printf("%s", TokenName(f));
+	    fprintf(stderr, "%s", TokenName(f));
 	if ((x = n->x.f.b) || f == '+' || f == '-')
-	    printf("%d ", x);
+	    fprintf(stderr, "%d ", x);
 	if (n->body)
 	    DumpGenericNode(n->body);
     }
@@ -828,9 +839,9 @@ static void DumpGenericNode(node n) {
 static void DumpPlusList(node l) {
     for (; l; l = l->next) {
 	if (l->name)
-	    printf("\"%s\" ", l->name);
+	    fprintf(stderr, "\"%s\" ", l->name);
 	DumpGenericNode(l->body);
-	printf("\n");
+	fprintf(stderr, "\n");
     }
 }
 
@@ -838,9 +849,9 @@ static void DumpFuncNode(node n) {
     if (!n)
 	return;
     
-    printf("%s \"%s\" (\n", TokenName(ADDTOFUNC), n->name);
+    fprintf(stderr, "%s \"%s\" (\n", TokenName(ADDTOFUNC), n->name);
     DumpPlusList(n->body);
-    printf(")\n");
+    fprintf(stderr, ")\n");
 }
 
 
@@ -848,28 +859,28 @@ static void DumpMenuNode(node n) {
     if (!n)
 	return;
     
-    printf("%s \"%s\" (\n", TokenName(ADDTOMENU), n->name);
+    fprintf(stderr, "%s \"%s\" (\n", TokenName(ADDTOMENU), n->name);
     DumpPlusList(n->body);
-    printf(")\n");
+    fprintf(stderr, ")\n");
 }
 
 static void DumpScreenNode(node n) {
     if (!n)
 	return;
     
-    printf("%s \"%s\"\n%s \"%s\" ", TokenName(ADDSCREEN), n->name,
+    fprintf(stderr, "%s \"%s\"\n%s \"%s\" ", TokenName(ADDSCREEN), n->name,
 	   TokenName(BACKGROUND), n->name);
     DumpColorName(n->x.color);
-    printf("(\n");
+    fprintf(stderr, "(\n");
     DumpNameList(n->body, TRUE);
-    printf(")\n");
+    fprintf(stderr, ")\n");
 }
 
 static void DumpBorderNode(node n) {
     if (!n)
 	return;
     
-    printf("%s \"%s\" %s (\n"
+    fprintf(stderr, "%s \"%s\" %s (\n"
 	   "\"%.3s\"\n"
 	   "\"%.3s\"\n"
 	   "\"%.3s\"\n"
@@ -881,26 +892,27 @@ static void DumpBorderNode(node n) {
 static void DumpKeyNode(node n) {
     if (!n)
 	return;
-    printf("%s %s ", TokenName(KEY), n->name);
+    fprintf(stderr, "%s %s ", TokenName(KEY), n->name);
     DumpGenericNode(n->body);
-    printf("\n");
+    fprintf(stderr, "\n");
 }
 
 static void DumpMouseNode(node n) {
-    byte buttons[4] = "\0\0\0", *b = buttons;
+    byte buttons[BUTTON_N_MAX + 1], *b = buttons;
+    udat i;
     
     if (!n)
 	return;
-    if (n->id & HOLD_LEFT)
-	*b++ = '1';
-    if (n->id & HOLD_MIDDLE)
-	*b++ = '2';
-    if (n->id & HOLD_RIGHT)
-	*b++ = '3';
     
-    printf("%s %s %s ", TokenName(MOUSE), buttons, n->name);
+    for (i = 0; i < BUTTON_N_MAX; i++) {
+	if (n->id & HOLD_N(i))
+	    *b++ = '1' + i;
+    }
+    WriteMem(b, '\0', BUTTON_N_MAX + 1 - (b - buttons));
+    
+    fprintf(stderr, "%s %s %s ", TokenName(MOUSE), buttons, n->name);
     DumpGenericNode(n->body);
-    printf("\n");
+    fprintf(stderr, "\n");
 }
 
 static void DumpGlobals(void) {
@@ -926,7 +938,7 @@ static void DumpGlobals(void) {
 
     for (l = CallList; l; l = l->next) {
 	DumpGenericNode(l);
-	printf("\n");
+	fprintf(stderr, "\n");
     }
 }
 
@@ -1026,7 +1038,7 @@ static byte CreateNeededScreens(node list, screen *res_Screens) {
 		    r = attr + w * h;
 		    while (len--) {
 #ifdef CONF__UNICODE
-			f = Tutf_IBM437_to_UTF_16[*n++];
+			f = Tutf_CP437_to_UTF_16[*n++];
 			*r++ = HWATTR(c, f);
 #else
 			*r++ = HWATTR(c, *n);
@@ -1227,11 +1239,8 @@ static byte ReadGlobals(void) {
     UpdateVisibleScreens(new_Screens);
     DeleteUnneededScreens(M[ScreenIndex]);
     
-    
-    if (!GlobalsAreStatic) {
-	shm_TSR();
+    if (!GlobalsAreStatic)
 	FreeMem(MenuBinds);
-    }
     GlobalsAreStatic = FALSE;
     
     if (All->CommonMenu)
@@ -1257,63 +1266,33 @@ static byte ReadGlobals(void) {
     
     All->SetUp->Flags |= GlobalFlags[0];
     All->SetUp->Flags ^= GlobalFlags[1];
-    All->SetUp->SelectionButton = GlobalFlags[2];
-    All->SetUp->PasteButton = GlobalFlags[3];
+    All->SetUp->ButtonSelection = GlobalFlags[2];
+    All->SetUp->ButtonPaste = GlobalFlags[3];
     All->SetUp->DeltaXShade = GlobalShadows[0];
     All->SetUp->DeltaYShade = GlobalShadows[1];
 
+    shm_TSR();
+
     QueuedDrawArea2FullScreen = TRUE;
     
     return TRUE;
 }
 
-/*
- * search for a file relative to HOME, to CONF_DESTDIR or as path
- * 
- * this for example will search "foo"
- * as "{HOME}/foo", "{CONF_DESTDIR}/lib/twin/foo" or plain "foo"
- */
-static str findfile(str name, uldat *fsize) {
-    str path;
-    byte CONST *dir;
-    byte CONST *search[3] = { HOME, conf_destdir_lib_twin, "" };
-    int i, min_i, max_i, len, nlen = strlen(name);
-    struct stat buf;
-    
-    if (flag_secure)
-	min_i = max_i = 1; /* only conf_destdir_lib_twin */
-    else
-	min_i = 0, max_i = 2;
-
-    for (i = min_i; i <= max_i && (dir = search[i]); i++) {
-	len = strlen(dir);
-	if ((path = AllocMem(len + nlen + 2))) {
-	    sprintf(path, "%s%s%s", dir, *dir ? "/" : "", name);
-	    if (stat(path, &buf) == 0) {
-		if (fsize)
-		    *fsize = buf.st_size;
-		return path;
-	    }
-	    FreeMem(path);
-	}
-    }
-    return NULL;
-}
-
 static byte rcparse(str path);
 
-#ifdef CONF_THIS_MODULE
+#ifdef THIS_MODULE
 static
 #endif
 byte rcload(void) {
     str path;
     uldat len;
 #ifndef DEBUG_FORK
-    int fds[2];
+    int fdm[2];
+    int fdl[2];
 #endif
     byte c = FALSE;
 
-    if (!(path = findfile(".twinrc", &len)))
+    if (!(path = FindFile(".twinrc", &len)))
 	return c;
 
     /*
@@ -1355,29 +1334,40 @@ byte rcload(void) {
     }
     return c;
 #else
-    if (socketpair(AF_UNIX, SOCK_STREAM, 0, fds) == 0) {
-	switch (fork()) {
-	  case -1: /* error */
-	    close(fds[0]);
-	    close(fds[1]);
-	    break;
-	  case 0:  /* child */
-	    QuitSignals();
-	    close(fds[0]);
-	    ClearGlobals();
-	    if (rcparse(path)) {
-		WriteGlobals();
-		shm_send(fds[1]);
+    if (socketpair(AF_UNIX, SOCK_STREAM, 0, fdm) == 0) {
+	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fdl) == 0) {
+	    switch (fork()) {
+	      case -1: /* error */
+		close(fdm[0]); close(fdm[1]);
+		close(fdl[0]); close(fdl[1]);
+		break;
+	      case 0:  /* child */
+		QuitSignals();
+		close(fdm[0]); close(fdl[0]);
+		if (fdl[1] != 2) {
+		    close(2);
+		    dup2(fdl[1], 2);
+		    close(fdl[1]);
+		}
+		ClearGlobals();
+		if (rcparse(path)) {
+		    WriteGlobals();
+		    shm_send(fdm[1]);
+		}
+		exit(0);
+		break;
+	      default: /* parent */
+		FreeMem(path);
+		close(fdm[1]); close(fdl[1]);
+		printk_receive_fd(fdl[0]);
+		
+		if (shm_receive(fdm[0]) && shm_shrink())
+		    c = ReadGlobals();
+		close(fdm[0]); close(fdl[0]);
+		break;
 	    }
-	    exit(0);
-	    break;
-	  default: /* parent */
-	    FreeMem(path);
-	    close(fds[1]);
-	    if (shm_receive(fds[0]) && shm_shrink())
-		c = ReadGlobals();
-	    close(fds[0]);
-	    break;
+	} else {
+	    close(fdm[0]); close(fdm[1]);
 	}
     }
     if (!c)
@@ -1387,9 +1377,7 @@ byte rcload(void) {
 }
 
 
-#ifdef CONF_THIS_MODULE
-
-MODULEVERSION;
+#ifdef THIS_MODULE
 
 byte InitModule(module Module) {
     Module->Private = (void *)rcload;
@@ -1398,5 +1386,5 @@ byte InitModule(module Module) {
 
 void QuitModule(module Module) {
 }
-#endif
+#endif /* THIS_MODULE */
 
