@@ -1,3 +1,11 @@
+/* Copyright (C) 2000 by Massimiliano Ghilardi
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ */
 
 
 #define TWScase(objtype,field,fieldtype) \
@@ -81,8 +89,8 @@
 
 
 
-static tobj *sockAllocListDeltaObjs(obj F, uldat *len, udat fdelta) {
-    uldat L = 0;
+static tobj *sockAllocListDeltaObjs(obj F, topaque *len, udat fdelta) {
+    topaque L = 0;
     tobj *_LW, *LW;
     obj O = F;
     while (O) {
